/*
 * Created on 27.12.2004
 *
 */
package io;

/**
 * @author Klaus Maier
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class Dataset 
{
	private String xTitle; // the identifier for the x-Axis, e.g. "time"
	private String yTitle; // the identifier for the y-Axis, e.g. "concentration"
	private String xUnit; // the units of the x-Axis, e.g. "seconds"
	private String yUnit; // the units of the y-Axis, e.g. "mol/l"
	private String title; // the title of this dataset, e.g. the name of the model
	
	private String[] identifiers; // the identifiers, e.g. different species

	private double[][] yVals; // the values
	private double[] xVals; // the sample points, e.g. time points
	private double[] zVals;
	
	private double maxX;
	private double minX;
	private double maxY;
	private double minY;

	public Dataset(int[][] yVals, 
				   int[] xVals, 
				   String xAxisTitle, 
				   String yAxisTitle, 
				   String xAxisUnit, 
				   String yAxisUnit, 
				   String[] identifiers, 
				   String title)
	{
		double[] xVal = this.intToDouble(xVals);
		double[][] yValues = this.intToDouble(yVals);		
				
		this.init(title, 
				  xVal, 
				  null,
				  yValues, 
				  identifiers, 
				  xTitle, 
				  yTitle, 
				  xUnit, 
				  yUnit);
	}	
	
	public Dataset(int[][] yVals, 
			   double[] xVals, 
			   String xAxisTitle, 
			   String yAxisTitle, 
			   String xAxisUnit, 
			   String yAxisUnit, 
			   String[] identifiers, 
			   String title)
{
	double[][] yValues = this.intToDouble(yVals);		
			
	this.init(title, 
				xVals, 
				null,
				yValues, 
				identifiers, 
				xTitle, 
				yTitle, 
				xUnit, 
				yUnit);
}	

	public Dataset(double[][] yVals, 
				   double[] xVals, 
				   String xTitle, 
				   String yTitle, 
				   String xUnit, 
				   String yUnit, 
				   String[] identifiers, 
				   String title)
	{				
		this.init(title, 
				  xVals, 
				  null,
				  yVals, 
				  identifiers, 
				  xTitle, 
				  yTitle, 
				  xUnit, 
				  yUnit);
	}

	public Dataset(double[] yVals, 
				   double[] xVals, 
				   String xTitle, 
				   String yTitle, 
				   String xUnit, 
				   String yUnit, 
				   String[] identifiers, 
				   String title)
	{
		double[][] yValues = new double[1][yVals.length];
		System.arraycopy(yVals, 0, yValues[0], 0, yVals.length);
		this.init(title, 
				  xVals, 
				  null,
				  yValues, 
				  identifiers, 
				  xTitle, 
				  yTitle, 
				  xUnit, 
				  yUnit);
	}

	public Dataset(int[] yVals, 
				   double[] xVals, 
				   String xTitle, 
				   String yTitle)
	{
		double[][] yValues = new double[1][yVals.length];
		double[] yVal = this.intToDouble(yVals);
		System.arraycopy(yVal, 0, yValues[0], 0, yVal.length);
		String[] ids = new String[yValues.length];
		this.init(title, 
				  xVals, 
				  null,
				  yValues, 
				  ids, 
				  xTitle, 
				  yTitle, 
				  "", 
				  "");
	}

	public Dataset(int[][] yVals, 
				   double[] xVals, 
				   String xTitle, 
				   String yTitle)
	{
		double[][] yValues = this.intToDouble(yVals);		
		String[] ids = new String[yValues.length];
		this.init(title, 
				  xVals, 
				  null,
				  yValues, 
				  ids, 
				  xTitle, 
				  yTitle, 
				  "", 
				  "");
	}

	public Dataset(double[][] yVals, 
			   	   double[] xVals,
				   String xTitle, 
				   String yTitle)
	{
		String[] ids = new String[yVals.length];
		this.init(title, 
				  xVals, 
				  null,
				  yVals, 
				  ids, 
				  xTitle, 
				  yTitle, 
				  "", 
				  "");
	}
	
	public Dataset(double[][] yVals, 
			   	   double[] xVals)
	{
		String[] ids = new String[yVals.length];
		this.init(title, 
				  xVals, 
				  null,
				  yVals, 
				  ids, 
				  "", 
				  "", 
				  "", 
				  "");
	}
	
	public Dataset(double[][] yVals, 
			   	   double[] xVals,
				   String[] ids)
	{
		this.init(title, 
				  xVals, 
				  null,
				  yVals, 
				  ids, 
				  "", 
				  "", 
				  "", 
				  "");
	}

	public Dataset(double[][] yVals, 
			double[] xVals,
			double[] zVals,
			String xTitle, 
			String yTitle)
	{
		String[] ids = new String[yVals.length];
		this.init(title, 
				  xVals, 
				  zVals,
				  yVals, 
				  ids, 
				  xTitle, 
				  yTitle, 
				  "", 
				  "");
	}

	public Dataset(int[][] yVals, 
			double[] xVals,
			double[] zVals,
			String xTitle, 
			String yTitle)
	{
		double[][] yValues = intToDouble(yVals);
		String[] ids = new String[yVals.length];
		this.init(title, 
				  xVals, 
				  zVals,
				  yValues, 
				  ids, 
				  xTitle, 
				  yTitle, 
				  "", 
				  "");
	}
	
	public Dataset(double[][] yVals, 
			double[] xVals,
			double[] zVals,
			String xTitle, 
			String yTitle,
			String[] ids)
	{
		this.init(title, 
				  xVals, 
				  zVals,
				  yVals, 
				  ids, 
				  xTitle, 
				  yTitle, 
				  "", 
				  "");
	}
	
	public Dataset(double[][] yVals, 
			double[] xVals,
			double[] zVals)
	{
		String[] ids = new String[yVals.length];
		this.init(title, 
				  xVals, 
				  zVals,
				  yVals, 
				  ids, 
				  "", 
				  "", 
				  "", 
				  "");
	}
	
	public Dataset(double[][] yVals, 
			double[] xVals,
			double[] zVals,
			String[] ids)
	{
		this.init(title, 
				  xVals, 
				  zVals,
				  yVals, 
				  ids, 
				  "", 
				  "", 
				  "", 
				  "");
	}

	private void init(String title, 
					  double[] xVals, 
					  double[] zVals, 
					  double[][] yVals, 
					  String[] identifiers, 
					  String xAxisTitle,
					  String yAxisTitle,
					  String xAxisUnit,
					  String yAxisUnit)
	{	
		this.zVals = zVals;
		this.xVals = xVals;
		this.yVals = yVals;
		this.identifiers = identifiers;
		this.xTitle = xAxisTitle;
		this.yTitle = yAxisTitle;
		this.xUnit = xAxisUnit;
		this.yUnit = yAxisUnit;
		this.title = title;
		this.getMaxMin();		
	}	
	
	/**
	 * determines the maximal and minimal x- and y- values
	 */
	private void getMaxMin()
	{
		this.maxX = Double.MIN_VALUE;
		this.maxY = Double.MIN_VALUE;
		this.minX = Double.MAX_VALUE;
		this.minY = Double.MAX_VALUE;

		for (int i = 0; i < this.yVals.length; i++)
		{
			for (int ii = 0; ii < this.yVals[i].length; ii++)
			{				
				if (this.xVals[ii] < this.minX)
						this.minX = this.xVals[ii];	
				if (this.xVals[ii] > this.maxX)
						this.maxX = this.xVals[ii];							
				if (this.yVals[i][ii] < this.minY)
						this.minY = this.yVals[i][ii];							
				if (this.yVals[i][ii] > this.maxY)
						this.maxY = this.yVals[i][ii];					
			}
		}
	}
	
	/**
	 * @return Returns the identifiers.
	 */
	public String[] getIdentifiers() {
		return identifiers;
	}
	
	/**
	 * @return Returns the title.
	 */
	public String getTitle() {
		return title;
	}
	
	/**
	 * @return Returns the xTitle.
	 */
	public String getXTitle() {
		return xTitle;
	}
	
	/**
	 * @return Returns the yTitle.
	 */
	public String getYTitle() {
		return yTitle;
	}
	
	/**
	 * @return Returns the xUnit.
	 */
	public String getXUnit() {
		return xUnit;
	}
	
	/**
	 * @return Returns the yUnit.
	 */
	public String getYUnit() {
		return yUnit;
	}
	
	/**
	 * @return Returns the xVals.
	 */
	public double[] getXVals() {
		return xVals;
	}
	
	/**
	 * @return Returns the yVals.
	 */
	public double[][] getYVals() {
		return yVals;
	}
	
	/**
	 * @return Returns the zVals.
	 */
	public double[] getZVals() {
		return zVals;
	}
	
	/**
	 * @return Returns the minX.
	 */
	public double getMinX() {
		return minX;
	}
	
	/**
	 * @return Returns the maxY.
	 */
	public double getMaxY() {
		return maxY;
	}
	
	/**
	 * @return Returns the maxX.
	 */
	public double getMaxX() {
		return maxX;
	}
	
	/**
	 * @return Returns the minY.
	 */
	public double getMinY() {
		return minY;
	}
	
	private double[][] intToDouble(int[][] array)
	{
		double[][] result = new double[array.length][];
		for (int i = 0; i < array.length; i++) 
		{
			result[i] = intToDouble(array[i]);
		}
		return result;
	}
	private double[] intToDouble(int[] array)
	{
		double[] result = new double[array.length];
		for (int i = 0; i < result.length; i++) 
		{
			result[i] = array[i];
		}	
		return result;
	}	
}
