/*
 * Created on Dec 11, 2004
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package math;


/**
 * @author kmaier
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class Echelon 
{
	private double tol = 1E-15;
	private int rank;
	
	public TMatrix getNr(TMatrix mat, double tol)
	{
		this.tol = tol;
		TMatrix echelon;
		TMatrix tm;
		TMatrix Nr;
		int c;
		
		tm = mat.getTranspose();
		
		echelon = mat.create(tm.getRowDimension(), tm.getColumnDimension());
		echelon = rank(tm, echelon, tol);
				
		// look for the unit vectors (r of them) in echelon to make up an 
		// identity matrix at the front of the echelon
		for (int i = 0; i < rank; i++) 
		{
			if (!ei_found(echelon, i, i))
			{
				c = find_ei(echelon,i);
//				System.out.println(i+"   "+(mat.getColumnDimension()-1)+"   "+c);
				echelon.exchangeCols(i,c);
				// swap the equivalent rows in the stoichiometry matrix to reorder
				mat.exchangeRows(i,c);
			}
		}		
				
		Nr = mat.create(rank, mat.getColumnDimension());
		// copy top rows into Nr
		for (int i = 0; i < rank; i++) 
		{
			for (int ii = 0; ii < mat.getColumnDimension(); ii++) 
			{
				Nr.set(i,ii,mat.get(i,ii));
			}
		}
		
		return Nr;
	}
	
	public TMatrix getL(TMatrix mat, double tol)
	{
		this.tol = tol;
		TMatrix echelon;
		TMatrix techelon;
		TMatrix tm;
		TMatrix L;
		int c;
		
//		tm = mat.getCopy();
//		tm.trans();
		tm = mat.getTranspose();
		
		echelon = mat.create(tm.getRowDimension(), tm.getColumnDimension());
		echelon = rank(tm, echelon, tol);
		
		// look for the unit vectors (r of them) in echelon to make up an 
		// identity matrix at the front of the echelon
		for (int i = 0; i < rank; i++) 
		{
			if (!ei_found(echelon, i, i))
			{
				c = find_ei(echelon,i);
				echelon.exchangeCols(i,c);
				// swap the equivalent rows in the stoichiometry matrix to reorder
				mat.exchangeRows(i,c);
			}
		}		
		
		techelon = echelon.getTranspose();
				
		// Now copy over the L part to eliminate redundant zero columns		
		L = techelon.create(techelon.getRowDimension(), rank);
		// copy top rows into Nr
		for (int i = 0; i < techelon.getRowDimension(); i++) 
		{
			for (int ii = 0; ii < rank; ii++) // only copy up to nRank columns
			{
				L.set(i,ii,techelon.get(i,ii));
			}
		}
		return L;
	}

	/**
	 * Find the rank r, of the matrix self. The reduced row echelon is returned in echelon. 
	 * tol is the magnitude of the largest number before it is assumed to be zero
	 * 
	 * @param echelon
	 * @param mat
	 * @param tol
	 */
	private TMatrix rank(TMatrix tm, TMatrix echelon, double tol)
	{
		int arow;
		int acol;
		int n;
		int m;
		int rowscan;
		
		double factor;
		
		arow = 0;
		acol = 0;
		
		n = tm.getRowDimension();
		m = tm.getColumnDimension();
		
		echelon = tm.getCopy();		
		
		do
		{
			if (isZero(echelon.get(arow,acol)))
			{
				// if the entry is zero work our way down the matrix
				// looking for a nonzero entry, when found swap it for arow
				rowscan = arow;
				do
				{
					// next row
					rowscan++;
					
					// Have we reached the end of the rows but we've still got columns left to scan
					if (rowscan >= n && acol < m-1)
					{
						rowscan = arow;
						acol++;
					}
					
					// if we've scanned the whole matrix, then let's get out..
					if (rowscan >= n)
					{
						echelon = cleanUpMatrix(echelon);
						rank = computeRank(echelon);
						return echelon;
					}
					
				}
				while(isZero(echelon.get(rowscan,acol))); // keep searching until non-zero entry
				
				// We've found a nonzero row entry so swap it with 
				// 'arow' which did have a zero as its entry 
				echelon.exchangeRows(arow,rowscan);
			}
			
			// arow now holds the row of interest
			factor = 1 / echelon.get(arow, acol);
			
			// reduce all the entries along the column by the factor
			for (int i = acol; i < m; i++) 
			{
				echelon.set(arow, i, echelon.get(arow, i) * factor);
			}
			
			// now eliminate all entries above and below arow, this generates the reduced form
			for (int i = 0; i < n; i++) 
			{
				// miss out arow itself 
				if (i!=arow && !(isZero(echelon.get(i,acol))))
				{
					factor = echelon.get(i, acol);
					// work your way along the column doing thew same operation
					for (int ii = acol; ii < m; ii++) 
					{
						echelon.set(i,ii,echelon.get(i,ii) - factor * echelon.get(arow,ii));
					}
				}
			}
			arow++;
			acol++;
		}
		while (arow < n && acol < m);
		
		echelon = cleanUpMatrix(echelon);		
		rank = computeRank(echelon);
		
		return echelon;
	}
	
	/**
	 * work out if the cth column of mat is made up of zeros except for a one at the rth row position
	 * 
	 * @param mat
	 * @param row
	 * @return
	 */
	private  boolean ei_found(TMatrix mat, int row, int column)
	{		
		for (int i = 0; i < mat.getRowDimension(); i++) 
		{
			if (i == row) // if we're at the chosen row then check if we've got a ONE
			{				
				if (!isOne(Math.abs(mat.get(i, column))))
				{
					return false;
				}				
			}
			else // if not then here should be a value of ZERO
			{
				if(!isZero(Math.abs(mat.get(i,column))))
				{
					return false;
				}
			}
		}		
		return true;
	}
		
	/**
	 * Search for a column, in the matrix mat, containing zeros throughout 
	 * except for the cth row which must be a ONE
	 * 
	 * @param mat
	 * @param c
	 * @return
	 */
	private int find_ei(TMatrix mat, int c)
	{		
		for (int i = 0; i < mat.getColumnDimension(); i++) 
		{
			// now scan the particular column we're on
		    if (ei_found(mat, c, i))
			{
				return i;
			}
		}		
		return -1;
	}
	
	/**
	 * internal routine that sets any values less than tol to 0.0
	 * 
	 * @param mat
	 */
	private TMatrix cleanUpMatrix(TMatrix mat)
	{
		// removes all numbers close to zero, i.e. between -tol and +tol
		for (int i = 0; i < mat.getRowDimension(); i++) 
		{
			for (int ii = 0; ii < mat.getColumnDimension(); ii++) 
			{
				if (isZero(mat.get(i,ii)))
					mat.set(i,ii,0);
			}
		}
		return mat;
	}
	
	
	/**
	 * works out the rank of a Matrix given the reduced row-echelon
	 * @param mat
	 * @return
	 */
	private int computeRank(TMatrix mat)
	{
		int rank = 0;		
		// find the rank brute force algorithm
		
		// search row by row for zero rows
		for (int i = 0; i < mat.getRowDimension(); i++) 
		{
			//search along the row looking for nonzero entry 
			for (int ii = 0; ii < mat.getColumnDimension(); ii++) 
			{
				if(!isZero(mat.get(i,ii)))
				{
					rank++;
					break;
				}
			}
		}		
		return rank;
	}
	
	
	private boolean isZero(double val)
	{
		double dval;
		long lval;		
		dval = val / this.tol;
		lval= Math.round(dval);
		dval= lval*this.tol;
		if (dval == 0.0)
			return true;
		return false;		
	}
	
	private boolean isOne(double val)
	{
		double dval;
		long lval;		
		dval = val / this.tol;
		lval= Math.round(dval);		
		dval= lval*this.tol;		
		if (dval == 1.0)
			return true;
		return false;	
	}
}
