/*
 * Created on Nov 24, 2004
 *
 */
package math.sde;

import Jama.Matrix;

/**
 * @author kmaier
 * 
 * This interface is used to define the methods an Model 
 * has to implement so that it can be simulated by a Langevin-type solver
 * 
 */
public interface SDE 
{	
	/**
	 * as a SDE is an ODE with an additional stochastic part, this method 
	 * is used to evaluate the SDE for the given values deterministically
	 * 
	 * @param vals
	 * @param x
	 * @return
	 */
	public double[] evalSDE(double[] vals, double x);	
	
	/**
	 * 
	 * this method calculates the stochastic part of a Langevin equation
	 * 
	 * @param vals
	 * @param x
	 * @return
	 */
	public double[] evalNoise(double[] vals, double x);
	
	/**
	 * returns the number of variables for this SDE;
	 * 
	 * @return
	 */
	public int getIndSpecNum();
	
	public int getDepSpecNum();
	
	/**
	 * returns the number of reactions for this SDE
	 * @return
	 */
	public int getReactNum();
	
	/**
	 * returns the Jacobian
	 * @param vals
	 * @return
	 */
	public Matrix getJacobian(double[] vals);
	
	/**
	 * returns the values for the depenedent species
	 * @param iSpeciesVals
	 * @return
	 */
	public double[] getDependentSpeciesValues(double[] iSpeciesVals);
}
