/*
 * Created on Dec 3, 2004
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package math.sde.solver;

import java.math.BigDecimal;

import math.sde.Langevin_Solver;
import math.sde.SDE;
import Jama.LUDecomposition;
import Jama.Matrix;

/**
 * @author kmaier
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class SemiImplicitEuler extends Langevin_Solver
{
	// the identity matrix
	Matrix identity;
	
	// the Jacobian 
//	JacobIF jacobian;
	
	// the stepsize for this solver
	double h;
	// the maximum error a iterative solution is allowed to have in the Newton-Raphson-Method, i.e. the tolerance
	double tol = 1E-10;	
	// the maximum number of iterations in the Newton-Raphson-Method
	int maxiter = 50;

	public SemiImplicitEuler(double h)
	{
		this.h = h;
	}

	public SemiImplicitEuler(double h, double tol, int iterations)
	{
		this.h = h;
		this.tol = tol;
		this.maxiter = iterations;
	}
	
	public double[][] solve(SDE sde, double x0, double[] timePoints, double[] startVals) throws Exception 
	{
		double[] oldVals = startVals;
		double[] nextVals = new double[startVals.length];
		double[] det;
		double[] noise;
		double[] iSpec = new double[sde.getIndSpecNum()];
		double[] dSpec = new double[sde.getDepSpecNum()];
		double hSqroot;
			
		double[][] result = new double[sde.getIndSpecNum() + sde.getDepSpecNum()][timePoints.length];      	     	
		double time = x0;
		int timePointCounter = 0;      
		double oldStepSize = h;      
		int counter;
		BigDecimal aBigDecimal;
		
		initIdentity(sde);
		
		if(x0 == timePoints[timePointCounter])
		{
			for (int i = 0; i < result.length; i++) 
			{
				result[i][0] = startVals[i];
			}
			timePointCounter++;
		}
		while (time < timePoints[timePoints.length - 1]) 
		{
			if ( (time + h) > timePoints[timePointCounter]) 
			{
				aBigDecimal = new BigDecimal(String.valueOf(timePoints[timePointCounter]));
				aBigDecimal = aBigDecimal.subtract(new BigDecimal(String.valueOf(time)));
				this.h = aBigDecimal.doubleValue();
			}     
			nextVals = newton(sde, oldVals, h, time);
			det = sde.evalSDE(nextVals, time);	
//			det = newton(sde, oldVals, h, time);
			noise = sde.evalNoise(oldVals, time);
			
			aBigDecimal = new BigDecimal(String.valueOf(time));
			aBigDecimal = aBigDecimal.add(new BigDecimal(String.valueOf(this.h)));      
			
			hSqroot = Math.sqrt(h);
			for (int j = 0; j < sde.getIndSpecNum(); j++) 
			{
				oldVals[j] += h * det[j] + hSqroot * noise[j];
				if (oldVals[j]<0) 
				{
					oldVals[j] = 0;
				}
				if (aBigDecimal.doubleValue() == timePoints[timePointCounter]) 
				{
					result[j][timePointCounter] = oldVals[j];		
//					System.out.println(oldVals[j]);
				}
				iSpec[j] = oldVals[j];
			}   
			dSpec = sde.getDependentSpeciesValues(iSpec);
			for (int j = sde.getIndSpecNum(); j < sde.getIndSpecNum()+sde.getDepSpecNum(); j++) 
			{	
				oldVals[j] = dSpec[j-sde.getIndSpecNum()];		
				if (oldVals[j]<0) 
				{
					oldVals[j] = 0;
				}
				if (aBigDecimal.doubleValue() == timePoints[timePointCounter]) 
				{
					result[j][timePointCounter] = oldVals[j];					  
				}
			}             
			if (aBigDecimal.doubleValue() == timePoints[timePointCounter]) 
			{	      
				timePointCounter++;
			}      
			time = aBigDecimal.doubleValue();
			h = oldStepSize;		  
		}			
		return result;
	}

	private double[] newton(SDE sde, double[] oldVals, double h, double time) throws Exception
	{		
		Matrix m_jacob = null;
		Matrix m_oldVals;
		Matrix m_new;
		Matrix m_inverse;
		Matrix m_temp1;
		Matrix m_temp2;
		Matrix m_solution;
		Matrix m_delta;
		
		double[] res;
		double[] newVals;
		double[] delta;
		double[] iSpec = new double[sde.getIndSpecNum()];
		double[] dSpec = new double[sde.getDepSpecNum()];
		
		boolean stop = false;		
		
		res = sde.evalSDE(oldVals, time);
		newVals = new double[oldVals.length];
		
		m_oldVals = new Matrix(oldVals, oldVals.length);
				
		for (int i = 0; i < sde.getIndSpecNum(); i++) 
		{
			newVals[i] = oldVals[i] + h * res[i];
			iSpec[i] = newVals[i];
		}
		dSpec = sde.getDependentSpeciesValues(iSpec);
		for (int i = sde.getIndSpecNum(); i < sde.getIndSpecNum()+sde.getDepSpecNum(); i++) 
		{
			newVals[i] = dSpec[i-sde.getIndSpecNum()];
		}
		
		for (int i = 0; i < this.maxiter; i++) 
		{
			m_new = new Matrix (newVals, oldVals.length);
			
			m_jacob = sde.getJacobian(newVals);
			m_inverse = invert(identity.minus(m_jacob.times(h)));
					
			m_temp1 = new Matrix(sde.evalSDE(newVals, time), oldVals.length).times(h);	
			m_temp2 = (m_new.minus(m_oldVals)).minus(m_temp1);
			
			m_delta = m_inverse.times(m_temp2.getMatrix(0,sde.getIndSpecNum()-1,0,0));
			
			stop = true;
						
			for (int ii = 0; ii < sde.getIndSpecNum(); ii++) 
			{
				newVals[ii] -= m_delta.get(ii,0);
				iSpec[ii] = newVals[ii];
				if (!(Math.abs(m_delta.get(ii,0))<=this.tol*Math.abs(newVals[ii])))
					stop = false;
			}
			dSpec = sde.getDependentSpeciesValues(iSpec);
			for (int ii = sde.getIndSpecNum(); ii < sde.getIndSpecNum()+sde.getDepSpecNum(); ii++) 
			{
				newVals[ii] = dSpec[ii-sde.getIndSpecNum()];
			}
			
			if (stop)
				break;
		}
		
//		if(!stop)
//			throw new Exception("Tolerance reached - System is probably too stiff");
				
		return newVals;
	}	
	
	private Matrix invert(Matrix jacob)
	{		
		LUDecomposition lu;
		Matrix vec;
		Matrix solVec;
		Matrix inverse;
		
		lu = jacob.lu();
		inverse = new Matrix(jacob.getRowDimension(), jacob.getRowDimension());
		vec = new Matrix(new double[jacob.getRowDimension()], jacob.getRowDimension());
		
		for (int i = 0; i < jacob.getRowDimension(); i++) 
		{
			for (int ii = 0; ii < jacob.getRowDimension(); ii++)
				vec.set(ii,0,0);
			vec.set(i,0,1);
			solVec = lu.solve(vec);
			for (int ii = 0; ii < jacob.getRowDimension(); ii++) 
				inverse.set(ii, i, solVec.get(ii, 0));
		}
		return inverse;
	}
		
	private void initIdentity(SDE sde)
	{
		identity = Matrix.identity(sde.getIndSpecNum(), sde.getIndSpecNum());
	}
}
