/*
 * Created on Jan 26, 2005
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package math.sto.solver.gibson;


/**
 * @author kmaier
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class IndexedPriorityQueue 
{
	final static int rootIndex = 0; // Heap tree root stored at index 0.
	final static int nullIndex = rootIndex - 1;
	int lastHeap = nullIndex; // Points to last Node in Heap. Initialize for empty heap.
	
	ReactionNode[] nodes;
	int[] index;
	
	public IndexedPriorityQueue (int reactionNum, double[] tau)
	{
		nodes = new ReactionNode[reactionNum];
		index = new int[reactionNum];
		lastHeap = reactionNum - 1;
		for (int i = 0; i < reactionNum; i++)
		{
			nodes[i] = new ReactionNode(i, tau[i]);
			index[i] = i;
		}
		buildHeap();
	}
	
	public ReactionNode getRoot()
	{
		return nodes[0];
	}
	
	// Swap the nodes at the indices a and b in the array.
	private void swap(int a, int b)	
	{	
		index[nodes[a].getReaction()] = b;
		index[nodes[b].getReaction()] = a;
		
		ReactionNode t = nodes[a];		
		nodes[a] = nodes[b];	     
		nodes[b] = t;	   
	}

	// heaporder the whole tree
	protected void buildHeap()	
	{ 
		// Start at last non-leaf node. Process all nodes in reverse storage order untill we reach the root
		// by this order, we ensure that heapify's precondition is fulfilled
		for (int k = lastInnerNode(); k != nullIndex; k=predesessor(k))
		{
		    heapify(k);
	    }
	 }

	// Heapify the subtree located at k
	// Precondition: k's both childtrees are heapordered
	protected void heapify(int k)
	{ 
		// move the key down the tree till we're done.
		while( k != nullIndex)
		{
			k = heapifyLocally(k);
		};     
	 }
	    
	// heap order Node k with respect to its both children
	// if keys had to be swapped return the node where k.key now is, nullIndex otherwise
	protected int heapifyLocally(int k)
	{
		int mc = minChild(k);	       
		if (mc == nullIndex){ return nullIndex;} // k is leaf. we're done.	       
		if (isGreaterThan(mc,k))
		{      
			return nullIndex;	  
		}
		else
		{	
			// if min child has smaller key then swap	         
			swap(k,mc);	         
			return mc;					      
		}	
	}		

	// return the bigger child of Node k. Return nullIndex if k is a leaf.	
	// precondition: heap must be complete	
	protected int minChild(int k)
	{
		int rc = rightChild(k);	    
		int lc = leftChild(k);	    
			    
		if (rc == nullIndex)
		{	    
			return lc;	     
		}
		else
		{ 
			// because heap is complete there must be a left child	        
			return isGreaterThan(lc, rc)? rc : lc;	     
		}
	}	

	// is Node at index a greater than Node at index b?
	protected boolean isGreaterThan(int a, int b)
	{
		return nodes[a].isGreaterThan(nodes[b]);
	}

	// return last inner Node (has no children) in Heap , nullIndex if there is no such node 	   
	protected int lastInnerNode()
	{	     
		return heapEmpty() ? nullIndex : parent(lastHeap);	   
	}
	   
	// is the heap empty?	   
	protected boolean heapEmpty()
	{	     
		return lastHeap == nullIndex;	   
	}
	
	// return the predessor of Node k in storage order, nullIndex if k is the root	   
	protected int predesessor(int k)	
	{	     
		return at(k - 1); 	  
	}
	
	public void update(int nodeNumber, double newTau)
	{
//		System.out.println(index[nodeNumber]+"    "+newTau);
		nodes[index[nodeNumber]].setTau(newTau);
		update_aux(index[nodeNumber]);
	}
	
	private void update_aux(int nodeNumber)
	{
		int minChild = nullIndex;
		int leftChild;
		int rightChild;
		int parent = parent(nodeNumber);
		if (parent != nullIndex)
			if (nodes[nodeNumber].getTau() < nodes[parent].getTau())
			{
				swap(nodeNumber, parent);
				update_aux(parent);
			}
			else
			{
				leftChild = leftChild(nodeNumber);
				if(leftChild != nullIndex)
					minChild = leftChild;
				rightChild = rightChild(nodeNumber);
				if (rightChild != nullIndex && minChild != nullIndex)
				{
					if(nodes[rightChild].getTau() < nodes[leftChild].getTau())
						minChild = rightChild;
				}
				else
				{
					if (rightChild != nullIndex)
						minChild = rightChild;
				}					
				
				if (minChild != nullIndex)
					if(nodes[minChild].getTau() < nodes[nodeNumber].getTau())
					{
						swap(nodeNumber, minChild);
						update_aux(minChild);
					}
			}
		else
		{
			leftChild = leftChild(nodeNumber);
			if(leftChild != nullIndex)
				minChild = leftChild;
			rightChild = rightChild(nodeNumber);
			if (rightChild != nullIndex)
				if(nodes[rightChild].getTau() < nodes[leftChild].getTau())
					minChild = rightChild;
			
			if (minChild != nullIndex)
				if(nodes[minChild].getTau() < nodes[nodeNumber].getTau())
				{
					swap(nodeNumber, minChild);
					update_aux(minChild);
				}
		}
	}
	

	// Return left child of Node k, nullIndex if there is no such child	
	private int leftChild(int k)	
	{	
		return at(2 * k + 1); // Because root is at 0. Root at 1 gives 2*k.	   
	}
	   
	
	// Return rihgt child of Node k, nullIndex if there is no such child	
	private int rightChild(int k)	
	{	
		return at(2 * k + 2); // Because root is at 0. Root at 1 gives 2*k +1.	   
	}
	   
	
	// return parent node of node k. return nullIndex if k is the root.	
	private int parent(int k)
	{	
		return k == rootIndex ? nullIndex : at((k-1) / 2); // Because root is at 0. Root at 1 gives k/2.	    
		// parent(root) is now root. This makes ?: necessary	   
	}	

	   
	// if index is in heap, return it unchanged, otherwise return nullIndex	   
	private int at(int index)
	{	     
		return index >= rootIndex && index <= lastHeap ? index : nullIndex;	   
	}	
}
