/*
 * Created on 02.11.2004
 */
package model;

/**
 * @author Klaus Maier
 *
 * A compartment represents a bounded space in which species are located. Compartments do not 
 * necessarily have to correspond to actual structures inside or outside a living cell, 
 * although models are often designed that way.
 */
public class Compartment implements Comparable
{
	String name; // used to give the compartment a unique identifier
	String outside; // the enclosing compartment
	String units;
	
	double volume;

	public Compartment (String name,
						String units,
						String outside,
						double volume)
	{
		this.name = name;
		this.outside = outside;
		this.units = units;
		this.volume = volume;
	}
	
	
	/**
	 * @return Returns the id.
	 */
	public String getName() {
		return name;
	}
	/**
	 * @return Returns the outside.
	 */
	public String getOutside() {
		return outside;
	}
	/**
	 * @return Returns the units.
	 */
	public String getUnits() {
		return units;
	}
	/**
	 * @return Returns the volume.
	 */
	public double getVolume() {
		return volume;
	}
	/**
	 * @param id The id to set.
	 */
	public void setName(String name) {
		this.name = name;
	}
	/**
	 * @param outside The outside to set.
	 */
	public void setOutside(String outside) {
		this.outside = outside;
	}
	/**
	 * @param units The units to set.
	 */
	public void setUnits(String units) {
		this.units = units;
	}
	/**
	 * @param volume The volume to set.
	 */
	public void setVolume(double volume) {
		this.volume = volume;
	}


	/* (non-Javadoc)
	 * @see model.Comparable#equals()
	 */
	public boolean equals(String name) 
	{
		if (this.name.equals(name))
			return true;
		return false;
	}


	/* (non-Javadoc)
	 * @see model.Comparable#isElementOf(model.Comparable[])
	 */
	public boolean isElementOf(Comparable[] array) 
	{
		for (int i = 0; i < array.length; i++) 
		{
			if (array[i].equals(name)) 
			{
				return true;
			}
		} 
		return false;
	}

	/**
	 * returns the first Index of this Comparable in the given Array.
	 * If not Element of this array the method will return -1
	 */
	public int getIndex(Comparable[] array) 
	{
		for (int i = 0; i < array.length; i++) 
		{
			if (array[i].equals(name)) 
			{
				return i;
			}
		} 
		return -1;
	}
	
}
