/*
 * Created on Mar 28, 2005
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package sim;

import java.util.LinkedList;
import java.util.List;

import model.Model;
import sbml.SBML_Reader;

import edu.caltech.sbw.*;

/**
 * @author kmaier
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public abstract class SimulatorBase 
{
	protected Model model;
	protected double[] vals;
	protected double t0;
	protected double tEnd;
	protected int numPoints;
	
	
	public abstract String getName();
	
	public abstract String getVersion();

	public abstract String getAuthor();

	public abstract String getDescription();

	public abstract String getDisplayName();

	public abstract String getCopyright();

	public abstract String getURL();

		 
	public void loadSBML(String sbml) throws SBWApplicationException
	{
		try 
		{
			model = SBML_Reader.load(sbml);
			if (Sys.OSIsWindows())
				model.initJarnac(sbml);
			reset();
		} 
		catch (Exception e) {
			throw new SBWApplicationException("Exception while loading the SBML", e.getMessage());
		}		
	}
	 
	public void setTimeStart(double t0)
	{
		this.t0 = t0;
	}
	
	public void setTimeEnd(double tEnd)
	{
		this.tEnd = tEnd;
	}
	
	public void setNumPoints(int numPoints)
	{
		this.numPoints = numPoints;
	}
	
	public void reset()
	{
		vals = new double[model.getIndSpecNum()+model.getDepSpecNum()];
		for (int i = 0; i < model.getIndSpecNum(); i++)
		{				
			vals[i] = (model.getISpecies()[i].getInitialAmount());
		}
		for (int i = model.getIndSpecNum(); i < model.getDepSpecNum()+model.getIndSpecNum(); i++)
		{
			vals[i] = (model.getDSpecies()[i - model.getIndSpecNum()].getInitialAmount());
		}			
	}
	
	public double[][] simulateEx(double t0, double tEnd, int points) throws SBWApplicationException, Exception
	{
		this.t0 = t0;
		this.tEnd = tEnd;
		this.numPoints = points;
		
		return simulate();
	}
	
	public void setFloatingSpeciesVector(double[] newVals)
	{
		vals = newVals;
	}
	
	public double[] getReactionRates() throws Exception
	{
		if (edu.caltech.sbw.Sys.OSIsWindows())
		{
		model.getJarnac().setFloatingSpeciesConcentrations(vals);
		return model.getJarnac().getReactionRates();
		}
		throw new Exception("not yet implemented.", null);
	}
	
	public double[] getRatesOfChange() throws Exception
	{
		if (edu.caltech.sbw.Sys.OSIsWindows())
		{
		model.getJarnac().setFloatingSpeciesConcentrations(vals);
		return model.getJarnac().getRatesOfChange();
		}
		throw new Exception("not yet implemented.", null);
	}
	
	public List getSpeciesNames()
	{
		LinkedList list = new LinkedList();

		for (int i = 0; i < model.getIndSpecNum(); i++) 
			list.addLast(model.getISpecies()[i].getName());
		for (int i = 0; i < model.getDepSpecNum(); i++) 
			list.addLast(model.getDSpecies()[i].getName());
		return list;
	} 

	public abstract double[][] simulate() throws SBWApplicationException, Exception;
				
	protected double[][] mergeResults(double[] time, double[][] values)
	{
//		double[][] res = new double[vals.length+1][time.length];
//		res[0] = time;
//		for (int i = 1; i < res.length; i++) 
//		{
//			res[i] = values[i-1];
//			vals[i-1] = values[i-1][values[i-1].length-1];
//		}
		double[][] res = new double[time.length][vals.length+1];
		int i = 0, j = 0;
		for (i = 0; i < time.length; i++)
		{
			res[i][0] = time[i];
			for (j = 0; j < vals.length; j++)
			{
				res[i][j+1] = values[j][i];
			}
		}
		return res;
	}
	protected double[] getTimePoints(double t0, double tEnd, int points)
	{
		double[] timePoints = new double[points];
		for (int i = 0; i < timePoints.length; i++) 
		{
			timePoints[i] = t0 + (i) * (tEnd -t0) / (points - 1);
		}		
		return timePoints;	
	}
}
